@@ -4,6 +4,10 @@ Module for interacting with an SQLite3 database (.db files)
 import csv
 import sqlite3
 import pathlib
+import logging
+import time
+import threading
+from queue import Queue
 
 from abc import ABC, abstractmethod
 from typing import List, Tuple, Union, Dict
@@ -13,6 +17,9 @@ from datetime import datetime, timedelta
 import warnings
 
 
+logger = logging.getLogger(__name__)
+
+
 class MissingColumnError(Exception):
     """
     Error for a missing column in a DatabaseTable
@@ -39,6 +46,67 @@ def get_db_column_names(file_path, table_name: str) -> List[str]:
     raise NotImplementedError('access the get_table_column_names method of DatabaseTable')
 
 
+class DatabaseManager:
+    _instances: List['DatabaseManager'] = []
+
+    CYCLE_TIME: float = 0.05
+
+    def __init__(self, target_file: str):
+        """
+        Database manager with insertion queue to avoid locked file issues
+
+        :param target_file: target database file
+        """
+        self.target_file = target_file
+        self._kill_flag = False
+        self.thread = threading.Thread(
+            target=self.insert_runner,
+            name='DI database thread',
+            daemon=True,
+        )
+        # queue expects an insert query and any arguments passed as the second variable
+        self.queue: Queue = Queue()
+        self.thread.start()
+        DatabaseManager._instances.append(self)
+
+    def insert_runner(self):
+        """Threading target for performing inserts into the database"""
+        logger.info('starting database insertion thread')
+        with sqlite3.connect(self.target_file, uri=True) as db:
+            while self._kill_flag is False:
+                with closing(db.cursor()) as cur:
+                    while self.queue.empty() is False:
+                        insert_query, values = self.queue.get()
+                        cur.execute(insert_query, values)
+                    db.commit()
+                time.sleep(self.CYCLE_TIME)
+        logger.info(f'database insertion thread terminating')
+
+    def stop_insertion_runner(self):
+        """killswitch for the insertion runner"""
+        self._kill_flag = True
+        DatabaseManager._instances.remove(self)
+
+    @classmethod
+    def get_database_manager(cls, file_path: str) -> 'DatabaseManager':
+        """
+        Retrieves a database manager instance for the provided file path. If an instance already exists for that
+        file path, the instance will be returned. Otherwise a new one will be created.
+
+        :param file_path: file path to write to
+        """
+        for inst in cls._instances:
+            if inst.target_file == file_path:
+                return inst
+        return cls(file_path)
+
+    @classmethod
+    def stop_all_instances(cls):
+        """stops all database insertion instance threads"""
+        for inst in cls._instances:
+            inst.stop_insertion_runner()
+
+
 class DatabaseTable(ABC):
     def __init__(self, file_path: Union[str, pathlib.Path]):
         """
@@ -51,6 +119,9 @@ class DatabaseTable(ABC):
 
         :param file_path: path to the database (.db file)
         """
+        file_path = pathlib.Path(file_path).absolute()
+        self.database_manager = DatabaseManager.get_database_manager(str(file_path.absolute()))
+
         self._file_path: pathlib.Path = None
         self.file_path = file_path
 
@@ -60,7 +131,7 @@ class DatabaseTable(ABC):
         for name, value_type in self.column_name_types:
             commands.append(f'{name} {value_type}')
         table_structure = ', '.join(commands)
-        with sqlite3.connect(self.file_path) as db:
+        with sqlite3.connect(str(self.file_path.absolute())) as db:
             with closing(db.cursor()) as cur:
                 # create the table
                 cur.execute(
@@ -130,12 +201,12 @@ class DatabaseTable(ABC):
     @property
     def db_table_names(self) -> List[str]:
         """A list of the name of all tables in the database"""
-        names = get_db_table_names(file_path=self.file_path)
+        names = get_db_table_names(file_path=str(self.file_path.absolute()))
         return names
 
     def get_table_column_names(self) -> List[str]:
         """Return a list of the column names for a table in a database, given the database path"""
-        with sqlite3.connect(self.file_path) as db:
+        with sqlite3.connect(str(self.file_path.absolute())) as db:
             with closing(db.cursor()) as cur:
                 # todo scrub for sql injection
                 cur.execute(self._select_query)
@@ -150,7 +221,7 @@ class DatabaseTable(ABC):
             list represents the value for a column index
         """
 
-        with sqlite3.connect(self.file_path) as db:
+        with sqlite3.connect(str(self.file_path.absolute())) as db:
             with closing(db.cursor()) as cur:
                 cur.execute(self._select_query)
                 rows = cur.fetchall()
@@ -175,11 +246,10 @@ class DatabaseTable(ABC):
         :param record: tuple or list of tuples. each tuple represents a record to add to the database table
         :return:
         """
-        with sqlite3.connect(self.file_path) as db:
-            with closing(db.cursor()) as cur:
-                for r in record:
-                    cur.execute(self._insert_query, r)
-                db.commit()
+        for r in record:
+            self.database_manager.queue.put(
+                (self._insert_query, r)
+            )
 
     def save_to_csv(self, file_path: Union[str, pathlib.Path] = None) -> pathlib.Path:
         """
@@ -195,7 +265,7 @@ class DatabaseTable(ABC):
         table_contents = self.retrieve()
 
         # create the csv file
-        with open(file_path, mode='wt', newline='') as csv_file:
+        with open(str(file_path.absolute()), mode='wt', newline='') as csv_file:
             csv_writer = csv.writer(csv_file)
             # write the heading row in the file
             column_names = self.get_table_column_names()
@@ -262,7 +332,7 @@ class TimeBasedDatabaseTable(DatabaseTable):
             'start_time': start_time,
             'end_time': end_time,
         }
-        with sqlite3.connect(self.file_path) as db:
+        with sqlite3.connect(str(self.file_path.absolute())) as db:
             with closing(db.cursor()) as cur:
                 cur.execute(command, values)
                 rows = cur.fetchall()
@@ -324,7 +394,7 @@ class TimeBasedDatabaseTable(DatabaseTable):
             'end_time': end_time,
         }
 
-        with sqlite3.connect(self.file_path) as db:
+        with sqlite3.connect(str(self.file_path.absolute())) as db:
             with closing(db.cursor()) as cur:
                 cur.execute(command, values)
 
@@ -358,7 +428,7 @@ class TimeBasedDatabaseTable(DatabaseTable):
                                        duration=duration)
 
         # create the csv file
-        with open(file_path, mode='wt', newline='') as csv_file:
+        with open(str(file_path.absolute()), mode='wt', newline='') as csv_file:
             csv_writer = csv.writer(csv_file)
             # write the heading row in the file
             column_names = self.get_table_column_names()
@@ -422,7 +492,7 @@ class DatabaseRetentionPolicy(ABC):
         :return: bool, whether the retention policy was run or not
         """
         if self.enabled:
-            with sqlite3.connect(database_table.file_path) as db:
+            with sqlite3.connect(str(database_table.file_path.absolute())) as db:
                 with closing(db.cursor()) as cur:
                     cur.execute(self._retention_policy, self._retention_policy_values)
             return True
