@@ -14,12 +14,12 @@ from typing import List, Union, Iterable, Callable
 from .timepoint import ActionTimePoint
 from .action import Action, ConfiguredAction
 from .sequencing import ConfiguredSequence
-from .mixin import InstanceRegistry
+from .mixin import InstanceRegistry, DumpableState
 
 logger = logging.getLogger(__name__)
 
 
-class SamplingScheduler(InstanceRegistry):
+class SamplingScheduler(InstanceRegistry, DumpableState):
     # pause time between cycle checks
     CYCLE_TIME = 0.5
 
