@@ -13,7 +13,7 @@ import sqlite3
 from contextlib import closing
 from datetime import datetime, timedelta
 
-from hein_control.db_tools import DatabaseTable, TimeBasedDatabaseTable, MissingColumnError, DatabaseRetentionPolicy
+from hein_control.db_tools import DatabaseTable, TimeBasedDatabaseTable, MissingColumnError, DatabaseRetentionPolicy, DatabaseManager
 
 lock = threading.Lock()
 
@@ -110,6 +110,13 @@ class TestDBTools(unittest.TestCase):
             Path(self.db_path).unlink()
         self.db_time_based_table = TestDatabaseTable(file_path=self.db_path)
 
+    def tearDown(self) -> None:
+        DatabaseManager.stop_all_instances()
+        time.sleep(0.5)
+        if Path(self.db_path).exists():
+            # delete any existing db files
+            Path(self.db_path).unlink()
+
     def test_db_extension(self):
         """tests db basic setting"""
         self.assertIsInstance(self.db_time_based_table.file_path, pathlib.Path, 'ensure table path is pathlib.Path')
@@ -118,8 +125,7 @@ class TestDBTools(unittest.TestCase):
     @run_locked
     def test_db_basic_properties(self):
         table_names = self.db_time_based_table.db_table_names
-        self.assertEqual(1, len(table_names), 'there should only be 1 table in the database')
-        self.assertEqual('TEST_DB_TABLE', table_names[0], 'the database table name should match')
+        self.assertIn('TEST_DB_TABLE', table_names, 'the table name should be contained in the db table')
 
         column_names = self.db_time_based_table.column_names
         self.assertEqual(4, len(column_names), 'there should only be 3 columns in the database')
@@ -137,6 +143,7 @@ class TestDBTools(unittest.TestCase):
         for i in range(5):
             expected_entries.append([f'text {i}', i, True if i % 2 == 0 else False])
             another_db_table.insert((f'text {i}', i, True if i % 2 == 0 else False))
+            time.sleep(0.5)
             retrieved = another_db_table.retrieve()
             self.assertEqual(expected_entries, retrieved,
                              'ensure all entries were inserted and retrieved properly')
@@ -147,6 +154,7 @@ class TestDBTools(unittest.TestCase):
         another_db_table.insert(*[(f'text {5}', 5, True if 5 % 2 == 0 else False),
                                   (f'text {6}', 6, True if 6 % 2 == 0 else False),
                                   ])
+        time.sleep(0.5)
         retrieved = another_db_table.retrieve()
         self.assertEqual(expected_entries, retrieved,
                          'ensure inserting multiple values at once works')
@@ -159,6 +167,7 @@ class TestDBTools(unittest.TestCase):
         insert_1_time = datetime.now()
         insert_1 = (insert_1_time, 'insert 1', 1, True)
         self.db_time_based_table.insert(insert_1)
+        time.sleep(0.5)
         expected_1 = [list(insert_1)]
         retrieve_1 = self.db_time_based_table.retrieve()
         self.assertEqual(expected_1, retrieve_1, 'ensure only 1 row with the correct values')
@@ -167,6 +176,7 @@ class TestDBTools(unittest.TestCase):
         insert_2_time = datetime.now()
         insert_2 = (insert_2_time, 'insert 2', 2, True)
         self.db_time_based_table.insert(insert_2)
+        time.sleep(0.5)
         expected_1_and_2 = [list(insert_1), list(insert_2)]
         retrieve_1_and_2 = self.db_time_based_table.retrieve()
         self.assertEqual(expected_1_and_2, retrieve_1_and_2, 'ensure both rows returned with the correct values')
@@ -185,6 +195,7 @@ class TestDBTools(unittest.TestCase):
         insert_5_time = datetime.now()
         insert_5 = (insert_5_time, 'insert 5', 5, False)
         self.db_time_based_table.insert(insert_5)
+        time.sleep(0.5)
         expected_1_and_2_and_3_and_4_and_5 = [list(insert_1), list(insert_2), list(insert_3), list(insert_4),
                                               list(insert_5)]
         retrieve_1_and_2_and_3_and_4_and_5 = self.db_time_based_table.retrieve()
@@ -200,7 +211,7 @@ class TestDBTools(unittest.TestCase):
 
         expected_2_and_3_and_4 = [list(insert_2), list(insert_3), list(insert_4)]
         # use duration of wait_time * 2.5 instead of 2 as a bit of a fudge factor for the time it takes to run code
-        retrieve_2_and_3_and_4 = self.db_time_based_table.retrieve(start_time=insert_2_time, duration=timedelta(seconds=wait_time * 2.5))
+        retrieve_2_and_3_and_4 = self.db_time_based_table.retrieve(start_time=insert_2_time, duration=timedelta(seconds=wait_time * 3))
         self.assertEqual(expected_2_and_3_and_4,
                          retrieve_2_and_3_and_4,
                          'ensure retrieving from a start time & duration works')
@@ -218,13 +229,13 @@ class TestDBTools(unittest.TestCase):
         self.assertEqual(expected_2_and_3_and_4_and_5, retrieve_2_and_3_and_4_and_5,
                          'ensure retrieving from a start time & end time works')
 
-        self.db_time_based_table.retain(start_time=insert_2_time, duration=timedelta(seconds=wait_time * 2.5))
+        self.db_time_based_table.retain(start_time=insert_2_time, duration=insert_4_time - insert_2_time)
         expected_2_and_3_and_4 = [list(insert_2), list(insert_3), list(insert_4)]
         retrieve_2_and_3_and_4 = self.db_time_based_table.retrieve()
         self.assertEqual(expected_2_and_3_and_4, retrieve_2_and_3_and_4,
                          'ensure retrieving from a start time & duration works')
 
-        self.db_time_based_table.retain(end_time=insert_4_time, duration=timedelta(seconds=wait_time * 1.5))
+        self.db_time_based_table.retain(end_time=insert_4_time, duration=insert_4_time - insert_3_time)
         expected_3_and_4 = [list(insert_3), list(insert_4)]
         retrieve_3_and_4 = self.db_time_based_table.retrieve()
         self.assertEqual(expected_3_and_4, retrieve_3_and_4,
@@ -234,6 +245,7 @@ class TestDBTools(unittest.TestCase):
         insert_31_days_ago_time = datetime.now() - timedelta(days=31, seconds=1)
         insert_31_days_ago = (insert_31_days_ago_time, 'insert 31 days ago', -31, True)
         self.db_time_based_table.insert(insert_31_days_ago)
+        time.sleep(0.5)
 
         retrieve_3_and_4 = self.db_time_based_table.retrieve()
         self.assertEqual(expected_3_and_4, retrieve_3_and_4,
@@ -275,6 +287,7 @@ class TestDBTools(unittest.TestCase):
         insert_32_days_ago_time = datetime.now() - timedelta(days=32)
         insert_32_days_ago = (insert_32_days_ago_time, 'insert 32 days ago', -32, True)
         self.db_time_based_table.insert(insert_32_days_ago)
+        time.sleep(0.5)
 
         # run the policy on the database table, but dont enable the policy so the values should not be removed
         expected_1_and_almost_31_and_31_and_32 = [list(insert_1), list(insert_almost_31_days_ago), list(insert_31_days_ago), list(insert_32_days_ago)]
@@ -336,7 +349,7 @@ class TestDBTools(unittest.TestCase):
 
         # ensure the default csv file path is set properly
         self.assertEqual(self.db_time_based_table._csv_path,
-                         Path(f'{self.db_path_without_file_type} - {self.db_time_based_table.table_name}.csv'),
+                         Path(f'{self.db_path_without_file_type} - {self.db_time_based_table.table_name}.csv').absolute(),
                          'ensure the default csv file path is set properly')
 
         # if any csv files from previous tests exist, remove them
@@ -372,6 +385,7 @@ class TestDBTools(unittest.TestCase):
             i = float(i)
             basic_db_table.insert((f'text {i}', i, True if i % 2 == 0 else False))
             self.db_time_based_table.insert((entry_time, f'text {i}', i, True if i % 2 == 0 else False))
+            time.sleep(0.5)
             # save the csv files
             basic_db_table.save_to_csv(basic_db_table_csv_path)
             self.db_time_based_table.save_to_csv()
