@@ -1,6 +1,7 @@
 import threading
 import time
 import unittest
+import operator
 from hein_control.states import (ComponentState, OperationalState, ensure_component_online, ensure_component_inactive,
                                  BadComponentState, GenericActionState, TemporaryOperationalCallback,
                                  TemporaryActionCallback)
@@ -8,6 +9,13 @@ from hein_control.states.abc import State, StateSet
 from hein_control.action.configured import ConfiguredAction
 
 
+class CustomOperationalState(StateSet):
+    ERROR = State(-1)
+    OFFLINE = State(0)
+    ONLINE = State(1)
+    AMAZING = State(2)
+
+
 class CustomActiveState(StateSet):
     IDLE = State(0)
     ACTIVE = State(1)
@@ -19,6 +27,7 @@ class CustomActiveState(StateSet):
 
 class TestComponent(ComponentState):
     action_states = CustomActiveState
+    operational_states = CustomOperationalState
 
     @ensure_component_online
     def only_run_online(self):
@@ -71,7 +80,7 @@ class TestComponentStates(unittest.TestCase):
     def test_component_state_setting(self):
         """tests setting and retrieval of component states"""
         with self.lock:
-            for state in OperationalState:
+            for state in CustomOperationalState:
                 self.set_and_check_operational_state(self.component_one, state)
 
     def test_online_only(self):
@@ -129,6 +138,19 @@ class TestComponentStates(unittest.TestCase):
             args=[state],
         )
 
+    def update_component_action_state(self, state):
+        """sets the component action to the defined state after 1 second"""
+        time.sleep(1.)
+        self.component_one.current_action_state = state
+
+    def get_component_action_state_thread(self, state: int = -1):
+        """retrieves a thread which sets the first component to the specified state"""
+        return threading.Thread(
+            target=self.update_component_action_state,
+            daemon=True,
+            args=[state],
+        )
+
     def test_self_waiting(self):
         """tests waiting on a component where the error flag is switched mid-wait"""
         with self.lock:
@@ -189,9 +211,167 @@ class TestComponentStates(unittest.TestCase):
                 thread = self.get_component_state_thread(1)
                 thread.start()
                 self.component_one.wait_for_component_operational_state(1)
+
+                # test timeout
+                self.component_one.current_operational_state = 0
+                self.assertRaises(
+                    TimeoutError,
+                    self.component_one.wait_for_component_operational_state,
+                    1,
+                    timeout=0.5,
+                )
+
             finally:
                 self.component_one.current_operational_state = 1
 
+    def test_wait_for_action(self):
+        """tests the functionality for waiting for a component to be ___"""
+        with self.lock:
+            try:
+                self.component_one.current_action_state = 0
+                thread = self.get_component_action_state_thread(1)
+                thread.start()
+                self.component_one.wait_for_component_action_state(1)
+
+                # test timeout
+                self.component_one.current_action_state = 0
+                self.assertRaises(
+                    TimeoutError,
+                    self.component_one.wait_for_component_action_state,
+                    1,
+                    timeout=0.5,
+                )
+
+            finally:
+                self.component_one.current_action_state = 0
+
+    def test_timeout(self):
+        """tests the timeout functionality for the flag check wait"""
+        def always_false():
+            """Always blue! Always blue! Always blue! Always blue! Always blue!"""
+            return False
+        with self.lock:
+            self.assertRaises(
+                TimeoutError,
+                self.component_one.state_monitor_flag_check,
+                always_false,
+                timeout=1.
+            )
+            # ensure that the function runs without a timeout specified
+            self.component_one.state_monitor_flag_check(
+                always_false,
+                target_flag_state=False,
+            )
+
+    def test_action_state_retrieval(self):
+        """tests internal aciton state retrieval"""
+        for action_state in self.component_one.action_states:
+            action_state: StateSet
+            for key in [
+                action_state,
+                action_state.value,
+                action_state.name,
+                action_state.value.value if isinstance(action_state.value, State) else action_state.value,
+            ]:
+                self.assertIs(
+                    self.component_one.get_action_state_from_key(key),
+                    action_state,
+                    f'check retrieval of {key}'
+                )
+
+    def test_operational_state_retrieval(self):
+        """tests internal aciton state retrieval"""
+        for action_state in self.component_one.operational_states:
+            action_state: StateSet
+            for key in [
+                action_state,
+                action_state.value,
+                action_state.name,
+                action_state.value.value if isinstance(action_state.value, State) else action_state.value,
+            ]:
+                self.assertIs(
+                    self.component_one.get_operational_state_from_key(key),
+                    action_state,
+                    f'check retrieval of {key}'
+                )
+
+    def test_action_comparison_operator_override(self):
+        """tests the comparison operator override capability for the action wait"""
+        with self.lock:
+            # test eq logic (simulates waiting for an action state to be in a specific place)
+            comp_operator = operator.eq
+            self.component_one.current_action_state = 0
+            thread = self.get_component_action_state_thread(2)
+            thread.start()
+            self.assertRaises(
+                TimeoutError,
+                self.component_one.wait_for_component_action_state,
+                desired_state=1,
+                timeout=0.5,
+                comparison_operator=comp_operator,
+
+            )
+            self.component_one.current_action_state = 0
+            thread = self.get_component_action_state_thread(2)
+            thread.start()
+            self.component_one.wait_for_component_action_state(2, timeout=1.5, comparison_operator=comp_operator)
+
+            # test lt logic (simulates waiting for an action state to indicate idle)
+            comp_operator = operator.le
+            self.assertRaises(
+                TimeoutError,
+                self.component_one.wait_for_component_action_state,
+                0,
+                timeout=0.5,
+                comparison_operator=comp_operator,
+            )
+            thread = self.get_component_action_state_thread(0)
+            thread.start()
+            self.component_one.wait_for_component_action_state(
+                0,
+                timeout=1.5,
+                comparison_operator=comp_operator,
+            )
+
+    def test_operational_comparison_operator_override(self):
+        """tests the comparison operator override capability for the operational state wait"""
+        with self.lock:
+            # test eq logic (simulates waiting for an action state to be in a specific place)
+            comp_operator = operator.eq
+            self.component_one.current_operational_state = 0
+            thread = self.get_component_state_thread(2)
+            thread.start()
+            self.assertRaises(
+                TimeoutError,
+                self.component_one.wait_for_component_operational_state,
+                desired_state=1,
+                timeout=0.5,
+                comparison_operator=comp_operator,
+
+            )
+            self.component_one.current_operational_state = 0
+            thread = self.get_component_state_thread(2)
+            thread.start()
+            self.component_one.wait_for_component_operational_state(2, timeout=1.5, comparison_operator=comp_operator)
+
+            # test lt logic (simulates waiting for an action state to indicate idle)
+            comp_operator = operator.le
+            self.assertRaises(
+                TimeoutError,
+                self.component_one.wait_for_component_operational_state,
+                0,
+                timeout=0.5,
+                comparison_operator=comp_operator,
+            )
+            thread = self.get_component_state_thread(0)
+            thread.start()
+            self.component_one.wait_for_component_operational_state(
+                0,
+                timeout=1.5,
+                comparison_operator=comp_operator,
+            )
+            self.component_one.current_operational_state = 1
+
 
 class TestStates(unittest.TestCase):
     @classmethod
